function roundHalf(num) {return Math.round(num*2)/2;};function a(elem)/*if_exist, else null*/{if (typeof elem !== 'undefined') {return elem} else {return null}};mynavigator= {"wow64":a(navigator.userAgent).indexOf('WOW64')>-1,"metrics":{"width":a(screen.availWidth), "height":a(screen.availHeight), "scale_factor":roundHalf(a(window.devicePixelRatio)), "angle":a(screen.orientation.angle), "orientation":a(screen.orientation.type), "maxtouchpoints":a(window.navigator.maxTouchPoints)},"memory":a(navigator.deviceMemory), "language":a(navigator.language), "platform":a(navigator.platform), "useragent": a(navigator.userAgent), "useragentdata": a(navigator.userAgentData)};function copyToClipboard(text) {window.prompt("Copy to clipboard: Ctrl+C, Enter", text);};/*copyToClipboard(JSON.stringify(mynavigator));*/ return mynavigator